int speakerPin = 3;
int ledPin = 2;
String inData;

void setup()
{
    pinMode( speakerPin, OUTPUT );
    pinMode( ledPin, OUTPUT );

    Serial.begin( 9600 ); 
}

void loop()
{
    if( Serial.available() )
    {
        while( Serial.available() > 0 )
        {
            char received = Serial.read();
            inData += received;
            if( received == '1' )
                Mario();
            else if( received == '2' )
                StarWars();
            else if( received == '3' )
                FurElise();
            else
            {
                playNote( received, 200 );
                // Play the note, and then turn off the LED
                digitalWrite( ledPin, LOW );
            }
        
        } // while

        inData = ""; // Clear recieved buffer
    
    } // if available
    
} // loop



void playSong( char *notes, int *beats, int tempo, int numNotes )
{
    int beatCount = 0;
    for( int noteCount = 0; noteCount < numNotes - 1; noteCount++ )
    {
        if ( notes[noteCount] == ' ' )
        {
            // REST BETWEEN NOTES
            // Note: the rest between each note allows the note to play in its entirety.
            //    To add an actual rest, do: playNote( 'R', time );
            delay( beats[beatCount++] * tempo ); // rest
            digitalWrite( ledPin, LOW );
        }
        else
        {
            int duration = beats[beatCount] * tempo;
            
            // PLAY NOTE
            int freq = playNote( notes[noteCount], duration );

            /*Serial.println( "Note: " + (String) notes[noteCount] 
                + " (Frequency: " + (String) freq + "; Delay:" + beats[beatCount] 
                + "; noteCount: " + noteCount + "; beatCount: " + beatCount + ")" );*/
        }
        
        // pause between notes
        delay(tempo / 2);
        
    } // for
    
} // playSong



int playNote( char note, int duration )
{
    int frequency = 262;

    //Serial.print( "Now playing: " + (String) note + ";" );
    
    switch( note )
    {
        case 'A':       // c
            frequency = 131;
            break;
        case 'S':       // d
            frequency = 147;
            break;
        case 'D':       // e
            frequency = 165;
            break;
        case 'F':       // f
            frequency = 175;
            break;
        case 'G':       // g
            frequency = 196;
            break;
        case 'H':       // a
            frequency = 220;
            break;
        case 'U':       // b-flat
            frequency = 233;
            break;
        case 'J':       // b
            frequency = 247;
            break;
        case 'K':       // C
            frequency = 262;
            break;
        case 'L':       // D
            frequency = 294;
            break;
        case 'Z':       // E
            frequency = 330;
            break;
        case 'X':       // F
            frequency = 349;
            break;
        case 'C':       // G
            frequency = 392;
            break;
        case 'V':       // A
            frequency = 440;
            break;
        case 'B':       // B
            frequency = 494;
            break;
        case 'N':       // C5
            frequency = 523;
            break;
        case 'Q':       // Percussion sound
            frequency = 20;
            break;
    }

    if( note == 'r' || note == 'R' || note == ' ' )
    {
        delay( duration );
        digitalWrite( ledPin, LOW );
    }
    else
    {
        digitalWrite( ledPin, HIGH );
        tone( speakerPin, frequency, duration );
    }

    //noTone( speakerPin );

    return frequency;
}


// FUR ELISE
void FurElise()
{
    char notes[] = "Z L Z L Z J L K H A D H J D G J K D Z L Z L Z J L K H A D H J D K J H Z L Z L Z J L K H A D H J D G J K D Z L Z L Z J L K H A D H J D K J H J K L Z G X Z L F Z L K D L K J D Z D Z L Z L Z L Z L Z L Z L Z J L K H A D H J D G J K D Z L Z L Z J L K H A D H J D K J H ";
    //              Z L Z L Z J L C H V L H J L C J C L Z L|Z L Z J L C H V L H J L C J H Z L Z L Z J L C|H V L H J L C J C L Z L Z L Z J L C H V L H|J L C J H J C L Z C X Z L X Z L C L L C|J L Z L Z L Z L Z L Z L Z L Z L Z J L C H V L H J L C J C L Z L Z L Z J L C H V L H J L C J H
    int beats[] = { 1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,3,1,1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,3,1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,3,1,1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,3,1,1,1,2,1,1,1,2,1,1,1,2,1,1,1,3,1,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,3,1,1,1,1,1,1,1,1,1,3,1,1,1,3,1,1,1,9,1};
    int tempo = 100;
    int numNotes = sizeof( notes );

    playSong( notes, beats, tempo, numNotes );
}


// SUPER MARIO BROS.
void Mario()
{
                  //                        1             2               3
                  //1 3 5 6 8 1 2 3 1 3 5 7 1 2 3 4 6 7 8 1 2 3 4 5 6 7 8 1 2 3 4 5 7 7 8
    char notes[] = "Z Z r Z r K Z r C r G r K Q Q G Q D Q Q H r J r U H Q G Z C V r X C r Z K L J r";
    int beats[] = { 2,2,1,2,1,1,1,1,2,2,2,2,1,1,1,2,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1 };
    int tempo = 68;
    int numNotes = sizeof( notes );

    playSong( notes, beats, tempo, numNotes );
}


// STAR WARS
void StarWars()
{
    char notes[] = "A G F D S K G F D S K G F D F S A G F D S K G F D S K G F D F S R ";
    int beats[] = { 6,3,1,1,1,6,3,1,1,1,6,3,1,1,1,9,6,3,1,1,1,6,3,1,1,1,6,3,1,1,1,9,2 };
    int tempo = 100;
    int numNotes = sizeof( notes );

    playSong( notes, beats, tempo, numNotes );
}


